@@ -12,7 +12,7 @@ All the Graph in this part should return data as following:
 
 Each Graph data (for each structure):
 
-``x``: Node feature matrix. np.ndarray, with shape [num_nodes, num_node_features]
+``x``: Node feature matrix. np.ndarray, with shape [num_nodes, nfeat_node]
 
 ``pos``: Node position matrix. np.ndarray, with shape [num_nodes, num_dimensions]
 
@@ -20,32 +20,33 @@ Each Graph data (for each structure):
 
 ``z``: atom numbers. np.ndarray, with shape [num_nodes,]
 
-``state_attr``: state feature. np.ndarray, shape (1, num_state_features)
+``state_attr``: state feature. np.ndarray, shape (1, nfeat_state)
 
 ``edge_index``: Graph connectivity in COO format. np.ndarray, with shape [2, num_edges] and type torch.long
 
-``edge_attr``: Edge feature matrix. np.ndarray,  with shape [num_edges, num_edge_features]
+``edge_attr``: Edge feature matrix. np.ndarray,  with shape [num_edges, nfeat_edge]
 
 Where the state_attr is added newly.
 
 """
-import itertools
-import math
+
 import os
+import warnings
 from shutil import rmtree
-from typing import List, Iterable, Union
+from typing import List, Iterable, Union, Tuple
 
 import ase.data as ase_data
 import numpy as np
 import torch
 from pymatgen.core import Structure, Element
+
 from pymatgen.optimization.neighbors import find_points_in_spheres
 from torch_geometric.data import Data
 
-from mgetool.tool import parallelize, batch_parallelize, time_this_function, tt
+from mgetool.tool import parallelize, batch_parallelize
 
 
-def data_merge(data_sub:List[Data], merge_key=None):
+def data_merge(data_sub: List[Data], merge_key=None):
     if merge_key is None:
         merge_key = ["state_attr", "edge_attr", "x"]
     data_base = data_sub[0]
@@ -63,10 +64,10 @@ def data_merge(data_sub:List[Data], merge_key=None):
                         v_base = getattr(data_base, k)
                         v_base_s = len(v_base.shape)
                         v_s = len(v.shape)
-                        if v_base_s==0 or v_s==0 or all((v_base_s==1,v_s==1)):
+                        if v_base_s == 0 or v_s == 0 or all((v_base_s == 1, v_s == 1)):
                             data_base.__setattr__(k, torch.cat([v_base.flatten(), v.flatten()]))
-                        elif all((v_base_s>=2,v_s>=2)) and v_base.shape[0]==v_base.shape[0]:
-                            data_base.__setattr__(k, torch.cat([v_base, v],dim=1))
+                        elif all((v_base_s >= 2, v_s >= 2)) and v_base.shape[0] == v_base.shape[0]:
+                            data_base.__setattr__(k, torch.cat([v_base, v], dim=1))
                         else:
                             data_base.__setattr__(k, v)
                     else:
@@ -74,6 +75,16 @@ def data_merge(data_sub:List[Data], merge_key=None):
     return data_base
 
 
+def data_dtype(data: Data) -> Data:
+    ks = data.keys
+    for k in ks:
+        v = getattr(data, k)
+        if isinstance(v, torch.Tensor):
+            if v.dtype == torch.float64:
+                data.__setattr__(k, v.float())
+    return data
+
+
 class StructureToData:
     def __init__(self, sub_converters: List = None, merge_key=None, n_jobs: int = 1,
                  batch_calculate: bool = True,
@@ -101,13 +112,38 @@ class StructureToData:
 
         """
         if self.sub_converters is None:
-            return self._convert(data, y=y, state_attr=state_attr, **kwargs)
+            data = self._convert(data, y=y, state_attr=state_attr, **kwargs)
         else:
-            data_sub = [ci.convert(data, y=y, state_attr=state_attr,**kwargs) for ci in self.sub_converters]
+            data_sub = [ci.convert(data, y=y, state_attr=state_attr, **kwargs) for ci in self.sub_converters]
             data = self._convert(data, y=y, state_attr=state_attr, **kwargs)
             data_sub.append(data)
-            data_base = data_merge(data_sub)
-            return data_base
+
+            data = data_merge(data_sub)
+            data = data_dtype(data)
+
+        return data
+
+    def _wrapper(self, *args, **kwargs):
+        try:
+            try:
+                con = self.convert(*args, **kwargs)
+            except TypeError as e:
+                print(e)
+                raise TypeError("Please check the above errors")
+
+            if isinstance(con, (List, Tuple)):
+                if len(con) == 2 and isinstance(con[1], bool):
+                    pass
+                else:
+                    con = (con, True)
+            else:
+                con = (con, True)
+            return con
+
+        except BaseException as e:
+            warnings.warn(e.__str__())
+            print(f"Bad conversion for:{args}, Check and index self.support_ to drop error data.", )
+            return np.nan, False
 
     def transform(self, structures: List[Structure], **kwargs) -> List[Data]:
         """
@@ -140,7 +176,7 @@ class StructureToData:
 
         try:
             kw = [{k: v[i] for k, v in kwargs.items()} for i in range(le)]
-        except  IndexError as e:
+        except IndexError as e:
             print(e)
             raise IndexError("Make sure the other parameters such as y and state_attr"
                              " are with same number (length) of structure.")
@@ -148,15 +184,19 @@ class StructureToData:
         iterables = zip(structures, kw)
 
         if not self.batch_calculate:
-            rets = parallelize(self.n_jobs, self.convert, iterables, tq=True, respective=True,
-                               respective_kwargs=True)
+            res = parallelize(self.n_jobs, self._wrapper(), iterables, tq=True, respective=True,
+                              respective_kwargs=True)
 
         else:
-            rets = batch_parallelize(self.n_jobs, self.convert, iterables, respective=True,
-                                     respective_kwargs=True,
-                                     tq=True, mode="j", batch_size=self.batch_size)
+            res = batch_parallelize(self.n_jobs, self._wrapper, iterables, respective=True,
+                                    respective_kwargs=True,
+                                    tq=True, mode="j", batch_size=self.batch_size)
 
-        return rets
+        ret, self.support_ = zip(*res)
+
+        ret = [i for i, j in zip(ret, self.support_) if j is True]
+
+        return ret
 
     def check_dup(self, structures, file_names="composition_name"):
         """Check the names duplication"""
@@ -184,7 +224,8 @@ class StructureToData:
             rmtree(raw_path)
         os.makedirs(raw_path)
 
-        result = self.transform(data, y=y, state_attr=state_attr,  **kwargs)
+        result = self.transform(data, y=y, state_attr=state_attr, **kwargs)
+
         print("Save raw files to {}.".format(raw_path))
         if save_mode in ["i", "r", "respective"]:
             fns = self.check_dup(data, file_names=file_names)
@@ -194,19 +235,21 @@ class StructureToData:
         print("Done.")
         return result
 
-
     @staticmethod
     def _convert(data: Structure, y=None, state_attr=None, **kwargs) -> Data:
         z = torch.tensor(list(data.atomic_numbers))
         pos = torch.from_numpy(data.cart_coords)
         if state_attr is not None:
-            state_attr = torch.tensor(state_attr)
+            state_attr = torch.tensor(state_attr, dtype=torch.float32)
+            state_attr = state_attr.reshape(1, 1) if len(state_attr.shape) == 0 else state_attr
         if y is not None:
-            y = torch.tensor(y)
+            y = torch.tensor(y).reshape(1, 1)
 
         for key, value in kwargs.items():
             try:
-                kwargs[key] = torch.tensor(value)
+                value = torch.tensor(value)
+                value = value.reshape(1, 1) if len(value.shape) == 0 else value
+                kwargs[key] = value
             except ValueError:
                 pass
 
@@ -252,11 +295,12 @@ class AddXPymatgen(StructureToData):
     def __init__(self, *args, prop_name=None, **kwargs):
         super(AddXPymatgen, self).__init__(*args, **kwargs)
         if prop_name is None:
-            self.prop_name = ["atomic_mass","average_ionic_radius","average_anionic_radius","atomic_radius_calculated"]
+            self.prop_name = ["atomic_mass", "average_ionic_radius", "average_anionic_radius",
+                              "atomic_radius_calculated"]
         self.da = [Element.from_Z(i) for i in range(1, 119)]
         self.da.insert(0, self.da[0])  # for start from 1
 
-    def _convert(self,data: Structure, **kwargs) -> Data:
+    def _convert(self, data: Structure, **kwargs) -> Data:
         x = torch.tensor([[getattr(self.da[i], pi) for pi in self.prop_name] for i in data.atomic_numbers])
         return Data(x=x)
 
@@ -267,7 +311,7 @@ class AddXASE(StructureToData):
 
     Get pymatgen element preprocessing.
     prop_name = [
-    'vdw_radii', 'reference_states', 'atomic_masses', 'covalent_radii'
+    'atomic_masses', 'covalent_radii'
     ]
     """
 
@@ -290,11 +334,11 @@ class AddXArray(StructureToData):
 
     """
 
-    def __init__(self, array:np.ndarray, *args, **kwargs):
+    def __init__(self, array: np.ndarray, *args, **kwargs):
         super(AddXArray, self).__init__(*args, **kwargs)
-        if array.ndim==1:
-            array = array.reshape(-1,1)
-        array = np.concatenate((array[0,:].reshape(1,-1),array),axis=0)
+        if array.ndim == 1:
+            array = array.reshape(-1, 1)
+        array = np.concatenate((array[0, :].reshape(1, -1), array), axis=0)  # add one line in 0.
         self.arrays = torch.from_numpy(array)
 
     def _convert(self, data: Structure, **kwargs) -> Data:
@@ -318,7 +362,7 @@ def _re_pbc(pbc: Union[bool, List[bool], np.ndarray], return_type="bool"):
     return pbc
 
 
-def _get_r_in_spheres(data,pbc=True, cutoff=5.0,numerical_tol=1e-6):
+def _get_r_in_spheres(data, pbc=True, cutoff=5.0, numerical_tol=1e-6):
     if isinstance(data, Structure):
         lattice_matrix = np.ascontiguousarray(np.array(data.lattice.matrix), dtype=float)
         if pbc is not False:
@@ -342,7 +386,7 @@ def _get_r_in_spheres(data,pbc=True, cutoff=5.0,numerical_tol=1e-6):
     center_indices = center_indices[exclude_self]
     neighbor_indices = neighbor_indices[exclude_self]
     distances = distances[exclude_self]
-    return center_indices, neighbor_indices, distances,None
+    return center_indices, neighbor_indices, distances, None
 
 
 def _get_xyz_in_spheres(data, pbc=True, cutoff=5.0, numerical_tol=1e-6):
@@ -373,7 +417,7 @@ def _get_xyz_in_spheres(data, pbc=True, cutoff=5.0, numerical_tol=1e-6):
     offset = np.dot(images, data.lattice.matrix)
 
     coords = cart_coords[neighbor_indices, :] - cart_coords[center_indices, :]
-    xyz = offset+coords
+    xyz = offset + coords
 
     return center_indices, neighbor_indices, distances, xyz
 
@@ -392,7 +436,6 @@ class AddPBCEdgeDistance(StructureToData):
     def __init__(self, *args, cutoff: float = 5.0,
                  numerical_tol: float = 1e-6,
                  pbc=True, **kwargs):
-
         """
         Get graph representations from structure within cutoff.
 
@@ -410,10 +453,11 @@ class AddPBCEdgeDistance(StructureToData):
         self.cutoff = cutoff
 
     def _convert(self, data: Structure, **kwargs) -> Data:
+        center_indices, neighbor_indices, distances, _ = _get_r_in_spheres(data, pbc=self.pbc,
+                                                                           cutoff=self.cutoff,
+                                                                           numerical_tol=self.numerical_tol)
 
-        center_indices,neighbor_indices,distances,_= _get_r_in_spheres(data, pbc=self.pbc,
-                                                                     cutoff=self.cutoff,
-                                                                     numerical_tol=self.numerical_tol)
+        assert len(set(center_indices)) == data.num_sites, f"Some atom are independent at cutoff: {self.cutoff}"
 
         edge_index = torch.vstack([torch.from_numpy(center_indices),
                                    torch.from_numpy(neighbor_indices)])
@@ -424,7 +468,7 @@ class AddPBCEdgeDistance(StructureToData):
 
 class AddPBCEdgeXYZ(StructureToData):
     """
-    Add x.
+    Add edge xyz.
     make sure.
 
     Get pymatgen element preprocessing.
@@ -436,7 +480,6 @@ class AddPBCEdgeXYZ(StructureToData):
     def __init__(self, *args, cutoff: float = 5.0,
                  numerical_tol: float = 1e-6,
                  pbc=True, **kwargs):
-
         """
         Get graph representations from structure within cutoff.
 
@@ -454,14 +497,16 @@ class AddPBCEdgeXYZ(StructureToData):
         self.cutoff = cutoff
 
     def _convert(self, data: Structure, **kwargs) -> Data:
-
         center_indices, neighbor_indices, distances, xyz = _get_xyz_in_spheres(data, pbc=self.pbc,
-                                                                     cutoff=self.cutoff,
-                                                                     numerical_tol=self.numerical_tol)
+                                                                               cutoff=self.cutoff,
+                                                                               numerical_tol=self.numerical_tol)
 
+        assert len(set(center_indices)) == data.num_sites, f"Some atom are independent at cutoff: {self.cutoff}"
+
+        assert center_indices.shape == neighbor_indices.shape
         edge_index = torch.vstack([torch.from_numpy(center_indices),
                                    torch.from_numpy(neighbor_indices)])
         edge_weight = torch.from_numpy(distances)
         edge_attr = torch.from_numpy(xyz)
 
-        return Data(edge_index=edge_index,edge_weight=edge_weight, edge_attr=edge_attr)
+        return Data(edge_index=edge_index, edge_weight=edge_weight, edge_attr=edge_attr)
