@@ -11,8 +11,7 @@ from torch.nn import Module
 from torch.optim.lr_scheduler import MultiStepLR
 from torch.utils.data import DataLoader
 
-from featurebox.models_geo.log_model import LogModule, make_dot_, HookGradientModule, get_layers_with_weight
-from featurebox.models_geo.general import getter_arr
+from pyg_extension.utils.log_model import LogModule, get_layers_with_weight, HookGradientModule, make_dot_
 
 
 def class_eval(prediction, target):
@@ -105,11 +104,11 @@ class LearningFlow:
         >>> val_loader = ...
 
 
-        >>>  model = CrystalGraphConvNet(num_node_features=91,
-        ...  num_edge_features=3,
-        ...  num_state_features=29,
-        ...  num_node_hidden_channels=128,
-        ...  num_node_interaction_channels=64,
+        >>>  model = CrystalGraphConvNet(nfeat_node=91,
+        ...  nfeat_edge=3,
+        ...  nfeat_state=29,
+        ...  nc_node_hidden=128,
+        ...  nc_node_interaction=64,
         ...  num_interactions=2,)
         >>> # model = CrystalGraphGCN(...)
         >>> # model = CrystalGraphGCN2(...)
@@ -130,13 +129,13 @@ class LearningFlow:
 
         >>> lf.run(50)
 
-    where the dataset could from :class:`featurebox.test_featurizers.base_graph_geo.StructureGraphGEO`
+    where the dataset could from StructureToData or :class:`featurebox.test_featurizers.base_graph_geo.StructureGraphGEO`
     """
 
-    def __init__(self, model: Module, train_loader: DataLoader, validate_loader: DataLoader, device: str = "cpu",
+    def __init__(self, model: Module, train_loader: DataLoader, validate_loader: DataLoader, device="cpu",
                  optimizer=None, clf: bool = False, loss_method=None, learning_rate: float = 1e-3, milestones=None,
-                 weight_decay: float = 0.0, checkpoint=True, scheduler=None, debug="", target_layers=(),
-                 loss_threshold: float = 0.1, print_freq: Union[int,str] = 10, print_what="all", process_label=None):
+                 weight_decay: float = 0.0, checkpoint=True, scheduler=None, debug: str = "", target_layers=(),
+                 loss_threshold: float = 0.1, print_freq: Union[int, str, None] = 10, print_what="all", process_label=None):
         """
 
         Parameters
@@ -301,10 +300,10 @@ class LearningFlow:
 
             self._train(epochi)
 
-            if self.debug is not False:
+            if "loop" in self.debug:
                 self.weight_log.get_weight(self.model)
 
-            if self.debug == "single":
+            if "single" in self.debug:
                 self.weight_log.stats()
 
             score = self._validate(epochi)
@@ -344,7 +343,7 @@ class LearningFlow:
                 vis_graph.render(format="pdf")
                 break
         if "hook" in self.debug:
-            self.hook_gradient = HookGradientModule(self.target_layers)
+            self.gradient_log = HookGradientModule(self.target_layers)
             self.model.train()
             for m, data in enumerate(self.train_loader):
                 data = data.to(self.device)
@@ -357,6 +356,7 @@ class LearningFlow:
                 lossi.backward(retain_graph=True)
                 # lossi.backward()
                 break
+            self.gradient_log.stats_single()
 
     def _train(self, epochi):
         self.model.train()
@@ -502,7 +502,7 @@ class LearningFlow:
         y_pre, y_true = self.predict(predict_loader, return_y_true=True, add_hook=False)
         return float(mae(y_pre, y_true))
 
-    def predict(self, predict_loader: DataLoader, return_y_true=False, add_hook=False, hook_layer_name=None):
+    def predict(self, predict_loader: DataLoader, return_y_true=False, add_hook=False, hook_layer=None):
         """
         Just predict by model,and add one forward hook to get put.
 
@@ -513,10 +513,8 @@ class LearningFlow:
         return_y_true:bool
             if return_y_true, return (y_preds, y_true)
         add_hook:bool
-            if add_hook, the model must contain torch native nn.ModuleList named ``fcs``
-            such as ``self.fcs = nn.ModuleList(...)`` in module.
-        hook_layer_name:str
-            user-defined hook layer name.
+        hook_layer:
+            layer of model.
 
         Returns
         -------
@@ -539,10 +537,7 @@ class LearningFlow:
                 def for_hook(module, input, output):
                     self.forward_hook_list.append(output.detach().cpu())
 
-                if hook_layer_name is None:
-                    raise NotImplementedError("The hook_layer_name is not passed")
-                else:
-                    handles.append(getter_arr(self.model, hook_layer_name).register_forward_hook(for_hook))
+                handles.append(hook_layer.register_forward_hook(for_hook))
             except BaseException as e:
                 print(e)
                 raise AttributeError("use ``hook_layer_name`` to defined the hook layer.")
